@@ -262,6 +262,7 @@ from .language import (
 # Execute GraphQL documents.
 from .execution import (
     execute,
+    execute_sync,
     default_field_resolver,
     default_type_resolver,
     get_directive_values,
@@ -320,6 +321,9 @@ from .validation import (
     UniqueFieldDefinitionNamesRule,
     UniqueDirectiveNamesRule,
     PossibleTypeExtensionsRule,
+    # Custom validation rules
+    NoDeprecatedCustomRule,
+    NoSchemaIntrospectionCustomRule,
 )
 
 # Create, format, and print GraphQL errors.
@@ -396,6 +400,8 @@ from .utilities import (
     DangerousChangeType,
     find_breaking_changes,
     find_dangerous_changes,
+    # Report all deprecated usages within a GraphQL document (deprecated).
+    find_deprecated_usages,
 )
 
 # Utilities for compatibility with the Python language.
@@ -606,6 +612,7 @@ __all__ = [
     "EnumTypeExtensionNode",
     "InputObjectTypeExtensionNode",
     "execute",
+    "execute_sync",
     "default_field_resolver",
     "default_type_resolver",
     "get_directive_values",
@@ -654,6 +661,8 @@ __all__ = [
     "UniqueFieldDefinitionNamesRule",
     "UniqueDirectiveNamesRule",
     "PossibleTypeExtensionsRule",
+    "NoDeprecatedCustomRule",
+    "NoSchemaIntrospectionCustomRule",
     "GraphQLError",
     "GraphQLSyntaxError",
     "located_error",
@@ -688,6 +697,7 @@ __all__ = [
     "is_valid_name_error",
     "find_breaking_changes",
     "find_dangerous_changes",
+    "find_deprecated_usages",
     "BreakingChange",
     "BreakingChangeType",
     "DangerousChange",
