@@ -1,4 +1,6 @@
-from asyncio import gather
+from asyncio import ensure_future, gather
+from collections.abc import Mapping
+from inspect import isawaitable
 from typing import (
     Any,
     Awaitable,
@@ -6,7 +8,6 @@ from typing import (
     Dict,
     Iterable,
     List,
-    NamedTuple,
     Optional,
     Set,
     Union,
@@ -68,6 +69,7 @@ __all__ = [
     "default_field_resolver",
     "default_type_resolver",
     "execute",
+    "execute_sync",
     "get_field_def",
     "ExecutionResult",
     "ExecutionContext",
@@ -94,19 +96,66 @@ __all__ = [
 # 3) inline fragment "spreads" e.g. "...on Type { a }"
 
 
-class ExecutionResult(NamedTuple):
+class ExecutionResult:
     """The result of GraphQL execution.
 
     - ``data`` is the result of a successful execution of the query.
     - ``errors`` is included when any errors occurred as a non-empty list.
+    - ``extensions`` is reserved for adding non-standard properties.
     """
 
+    __slots__ = "data", "errors", "extensions"
+
     data: Optional[Dict[str, Any]]
     errors: Optional[List[GraphQLError]]
+    extensions: Optional[Dict[str, Any]]
 
+    def __init__(
+        self,
+        data: Optional[Dict[str, Any]] = None,
+        errors: Optional[List[GraphQLError]] = None,
+        extensions: Optional[Dict[str, Any]] = None,
+    ):
+        self.data = data
+        self.errors = errors
+        self.extensions = extensions
+
+    def __repr__(self) -> str:
+        name = self.__class__.__name__
+        ext = "" if self.extensions is None else f", extensions={self.extensions}"
+        return f"{name}(data={self.data!r}, errors={self.errors!r}{ext})"
+
+    def __iter__(self) -> Iterable[Any]:
+        return iter((self.data, self.errors))
+
+    @property
+    def formatted(self) -> Dict[str, Any]:
+        """Get execution result formatted according to the specification."""
+        if self.extensions is None:
+            return dict(data=self.data, errors=self.errors)
+        return dict(data=self.data, errors=self.errors, extensions=self.extensions)
+
+    def __eq__(self, other: Any) -> bool:
+        if isinstance(other, dict):
+            if "extensions" not in other:
+                return other == dict(data=self.data, errors=self.errors)
+            return other == dict(
+                data=self.data, errors=self.errors, extensions=self.extensions
+            )
+        if isinstance(other, tuple):
+            if len(other) == 2:
+                return other == (self.data, self.errors)
+            return other == (self.data, self.errors, self.extensions)
+        return (
+            isinstance(other, self.__class__)
+            and other.data == self.data
+            and other.errors == self.errors
+            and other.extensions == self.extensions
+        )
+
+    def __ne__(self, other: Any) -> bool:
+        return not self == other
 
-# noinspection PyTypeHints
-ExecutionResult.__new__.__defaults__ = (None, None)  # type: ignore
 
 Middleware = Optional[Union[Tuple, List, MiddlewareManager]]
 
@@ -320,7 +369,7 @@ class ExecutionContext:
         results: AwaitableOrValue[Dict[str, Any]] = {}
         is_awaitable = self.is_awaitable
         for response_name, field_nodes in fields.items():
-            field_path = Path(path, response_name)
+            field_path = Path(path, response_name, parent_type.name)
             result = self.resolve_field(
                 parent_type, source_value, field_nodes, field_path
             )
@@ -345,7 +394,9 @@ class ExecutionContext:
             elif is_awaitable(result):
                 # noinspection PyShadowingNames
                 async def set_result(
-                    results: Dict[str, Any], response_name: str, result: Awaitable,
+                    results: Dict[str, Any],
+                    response_name: str,
+                    result: Awaitable,
                 ) -> Dict[str, Any]:
                     results[response_name] = await result
                     return results
@@ -379,7 +430,7 @@ class ExecutionContext:
         awaitable_fields: List[str] = []
         append_awaitable = awaitable_fields.append
         for response_name, field_nodes in fields.items():
-            field_path = Path(path, response_name)
+            field_path = Path(path, response_name, parent_type.name)
             result = self.resolve_field(
                 parent_type, source_value, field_nodes, field_path
             )
@@ -589,17 +640,13 @@ class ExecutionContext:
                 async def await_result() -> Any:
                     try:
                         return await result
-                    except GraphQLError as error:
-                        return error
                     except Exception as error:
-                        return GraphQLError(str(error), original_error=error)
+                        return error
 
                 return await_result()
             return result
-        except GraphQLError as error:
-            return error
         except Exception as error:
-            return GraphQLError(str(error), original_error=error)
+            return error
 
     def complete_value_catching_error(
         self,
@@ -778,7 +825,7 @@ class ExecutionContext:
         for index, item in enumerate(result):
             # No need to modify the info object containing the path, since from here on
             # it is not ever accessed by resolver functions.
-            field_path = path.add_key(index)
+            field_path = path.add_key(index, None)
             completed_item = self.complete_value_catching_error(
                 item_type, field_nodes, info, field_path, item
             )
@@ -1049,6 +1096,61 @@ def execute(
     return exe_context.build_response(data)
 
 
+def assume_not_awaitable(_value: Any) -> bool:
+    """Replacement for isawaitable if everything is assumed to be synchronous."""
+    return False
+
+
+def execute_sync(
+    schema: GraphQLSchema,
+    document: DocumentNode,
+    root_value: Any = None,
+    context_value: Any = None,
+    variable_values: Optional[Dict[str, Any]] = None,
+    operation_name: Optional[str] = None,
+    field_resolver: Optional[GraphQLFieldResolver] = None,
+    type_resolver: Optional[GraphQLTypeResolver] = None,
+    middleware: Optional[Middleware] = None,
+    execution_context_class: Optional[Type["ExecutionContext"]] = None,
+    check_sync: bool = False,
+) -> ExecutionResult:
+    """Execute a GraphQL operation synchronously.
+
+    Also implements the "Evaluating requests" section of the GraphQL specification.
+
+    However, it guarantees to complete synchronously (or throw an error) assuming
+    that all field resolvers are also synchronous.
+
+    Set check_sync to True to still run checks that no awaitable values are returned.
+    """
+    is_awaitable = (
+        check_sync
+        if callable(check_sync)
+        else (None if check_sync else assume_not_awaitable)
+    )
+
+    result = execute(
+        schema,
+        document,
+        root_value,
+        context_value,
+        variable_values,
+        operation_name,
+        field_resolver,
+        type_resolver,
+        middleware,
+        execution_context_class,
+        is_awaitable,
+    )
+
+    # Assert that the execution was synchronous.
+    if isawaitable(result):
+        ensure_future(cast(Awaitable[ExecutionResult], result)).cancel()
+        raise RuntimeError("GraphQL execution failed to complete synchronously.")
+
+    return cast(ExecutionResult, result)
+
+
 def assert_valid_execution_arguments(
     schema: GraphQLSchema,
     document: DocumentNode,
@@ -1116,7 +1218,7 @@ def invalid_return_type_error(
 
 def get_typename(value: Any) -> Optional[str]:
     """Get the ``__typename`` property of the given value."""
-    if isinstance(value, dict):
+    if isinstance(value, Mapping):
         return value.get("__typename")
     # need to de-mangle the attribute assumed to be "private" in Python
     for cls in value.__class__.__mro__:
@@ -1193,7 +1295,7 @@ def default_field_resolver(source: Any, info: GraphQLResolveInfo, **args: Any) -
     field_name = info.field_name
     value = (
         source.get(field_name)
-        if isinstance(source, dict)
+        if isinstance(source, Mapping)
         else getattr(source, field_name, None)
     )
     if callable(value):
