@@ -1,14 +1,45 @@
 from functools import wraps
 from json import dumps
-from typing import Collection, Optional
+from typing import Any, Callable, Collection, Optional
 
-from .ast import Node, OperationType
+from ..language.ast import Node, OperationType
 from .visitor import visit, Visitor
 from .block_string import print_block_string
 
 __all__ = ["print_ast"]
 
 
+Strings = Collection[str]
+
+
+class PrintedNode:
+    """A union type for all nodes that have been processed by the printer."""
+
+    alias: str
+    arguments: Strings
+    block: bool
+    default_value: str
+    definitions: Strings
+    description: str
+    directives: str
+    fields: Strings
+    interfaces: Strings
+    locations: Strings
+    name: str
+    operation: OperationType
+    operation_types: Strings
+    repeatable: bool
+    selection_set: str
+    selections: Strings
+    type: str
+    type_condition: str
+    types: Strings
+    value: str
+    values: Strings
+    variable: str
+    variable_definitions: Strings
+
+
 def print_ast(ast: Node) -> str:
     """Convert an AST into a string.
 
@@ -17,83 +48,93 @@ def print_ast(ast: Node) -> str:
     return visit(ast, PrintAstVisitor())
 
 
-def add_description(method):
-    """Decorator adding the description to the output of a visitor method."""
+def add_description(method: Callable[..., str]) -> Callable:
+    """Decorator adding the description to the output of a static visitor method."""
 
     @wraps(method)
-    def wrapped(self, node, *args):
-        return join([node.description, method(self, node, *args)], "\n")
+    def wrapped(node: PrintedNode, *args: Any) -> str:
+        return join((node.description, method(node, *args)), "\n")
 
     return wrapped
 
 
-# noinspection PyMethodMayBeStatic
 class PrintAstVisitor(Visitor):
-    def leave_name(self, node, *_args):
+    @staticmethod
+    def leave_name(node: PrintedNode, *_args: Any) -> str:
         return node.value
 
-    def leave_variable(self, node, *_args):
+    @staticmethod
+    def leave_variable(node: PrintedNode, *_args: Any) -> str:
         return f"${node.name}"
 
     # Document
 
-    def leave_document(self, node, *_args):
+    @staticmethod
+    def leave_document(node: PrintedNode, *_args: Any) -> str:
         return join(node.definitions, "\n\n") + "\n"
 
-    def leave_operation_definition(self, node, *_args):
+    @staticmethod
+    def leave_operation_definition(node: PrintedNode, *_args: Any) -> str:
         name, op, selection_set = node.name, node.operation, node.selection_set
         var_defs = wrap("(", join(node.variable_definitions, ", "), ")")
         directives = join(node.directives, " ")
         # Anonymous queries with no directives or variable definitions can use the
         # query short form.
         return (
-            join([op.value, join([name, var_defs]), directives, selection_set], " ")
+            join((op.value, join((name, var_defs)), directives, selection_set), " ")
             if (name or directives or var_defs or op != OperationType.QUERY)
             else selection_set
         )
 
-    def leave_variable_definition(self, node, *_args):
+    @staticmethod
+    def leave_variable_definition(node: PrintedNode, *_args: Any) -> str:
         return (
             f"{node.variable}: {node.type}"
             f"{wrap(' = ', node.default_value)}"
             f"{wrap(' ', join(node.directives, ' '))}"
         )
 
-    def leave_selection_set(self, node, *_args):
+    @staticmethod
+    def leave_selection_set(node: PrintedNode, *_args: Any) -> str:
         return block(node.selections)
 
-    def leave_field(self, node, *_args):
+    @staticmethod
+    def leave_field(node: PrintedNode, *_args: Any) -> str:
         return join(
-            [
+            (
                 wrap("", node.alias, ": ")
                 + node.name
                 + wrap("(", join(node.arguments, ", "), ")"),
                 join(node.directives, " "),
                 node.selection_set,
-            ],
+            ),
             " ",
         )
 
-    def leave_argument(self, node, *_args):
+    @staticmethod
+    def leave_argument(node: PrintedNode, *_args: Any) -> str:
         return f"{node.name}: {node.value}"
 
     # Fragments
 
-    def leave_fragment_spread(self, node, *_args):
+    @staticmethod
+    def leave_fragment_spread(node: PrintedNode, *_args: Any) -> str:
         return f"...{node.name}{wrap(' ', join(node.directives, ' '))}"
 
-    def leave_inline_fragment(self, node, *_args):
+    @staticmethod
+    def leave_inline_fragment(node: PrintedNode, *_args: Any) -> str:
         return join(
-            [
+            (
                 "...",
                 wrap("on ", node.type_condition),
                 join(node.directives, " "),
                 node.selection_set,
-            ],
+            ),
             " ",
         )
 
-    def leave_fragment_definition(self, node, *_args):
+    @staticmethod
+    def leave_fragment_definition(node: PrintedNode, *_args: Any) -> str:
         # Note: fragment variable definitions are experimental and may be changed or
         # removed in the future.
         return (
@@ -106,81 +147,99 @@ class PrintAstVisitor(Visitor):
 
     # Value
 
-    def leave_int_value(self, node, *_args):
+    @staticmethod
+    def leave_int_value(node: PrintedNode, *_args: Any) -> str:
         return node.value
 
-    def leave_float_value(self, node, *_args):
+    @staticmethod
+    def leave_float_value(node: PrintedNode, *_args: Any) -> str:
         return node.value
 
-    def leave_string_value(self, node, key, *_args):
+    @staticmethod
+    def leave_string_value(node: PrintedNode, key: str, *_args: Any) -> str:
         if node.block:
             return print_block_string(node.value, "" if key == "description" else "  ")
         return dumps(node.value)
 
-    def leave_boolean_value(self, node, *_args):
+    @staticmethod
+    def leave_boolean_value(node: PrintedNode, *_args: Any) -> str:
         return "true" if node.value else "false"
 
-    def leave_null_value(self, _node, *_args):
+    @staticmethod
+    def leave_null_value(_node: PrintedNode, *_args: Any) -> str:
         return "null"
 
-    def leave_enum_value(self, node, *_args):
+    @staticmethod
+    def leave_enum_value(node: PrintedNode, *_args: Any) -> str:
         return node.value
 
-    def leave_list_value(self, node, *_args):
+    @staticmethod
+    def leave_list_value(node: PrintedNode, *_args: Any) -> str:
         return f"[{join(node.values, ', ')}]"
 
-    def leave_object_value(self, node, *_args):
+    @staticmethod
+    def leave_object_value(node: PrintedNode, *_args: Any) -> str:
         return f"{{{join(node.fields, ', ')}}}"
 
-    def leave_object_field(self, node, *_args):
+    @staticmethod
+    def leave_object_field(node: PrintedNode, *_args: Any) -> str:
         return f"{node.name}: {node.value}"
 
     # Directive
 
-    def leave_directive(self, node, *_args):
+    @staticmethod
+    def leave_directive(node: PrintedNode, *_args: Any) -> str:
         return f"@{node.name}{wrap('(', join(node.arguments, ', '), ')')}"
 
     # Type
 
-    def leave_named_type(self, node, *_args):
+    @staticmethod
+    def leave_named_type(node: PrintedNode, *_args: Any) -> str:
         return node.name
 
-    def leave_list_type(self, node, *_args):
+    @staticmethod
+    def leave_list_type(node: PrintedNode, *_args: Any) -> str:
         return f"[{node.type}]"
 
-    def leave_non_null_type(self, node, *_args):
+    @staticmethod
+    def leave_non_null_type(node: PrintedNode, *_args: Any) -> str:
         return f"{node.type}!"
 
     # Type System Definitions
 
+    @staticmethod
     @add_description
-    def leave_schema_definition(self, node, *_args):
+    def leave_schema_definition(node: PrintedNode, *_args: Any) -> str:
         return join(
-            ["schema", join(node.directives, " "), block(node.operation_types)], " "
+            ("schema", join(node.directives, " "), block(node.operation_types)), " "
         )
 
-    def leave_operation_type_definition(self, node, *_args):
+    @staticmethod
+    def leave_operation_type_definition(node: PrintedNode, *_args: Any) -> str:
         return f"{node.operation.value}: {node.type}"
 
+    @staticmethod
     @add_description
-    def leave_scalar_type_definition(self, node, *_args):
-        return join(["scalar", node.name, join(node.directives, " ")], " ")
+    def leave_scalar_type_definition(node: PrintedNode, *_args: Any) -> str:
+        return join(("scalar", node.name, join(node.directives, " ")), " ")
 
+    @staticmethod
     @add_description
-    def leave_object_type_definition(self, node, *_args):
+    def leave_object_type_definition(node: PrintedNode, *_args: Any) -> str:
         return join(
-            [
+            (
                 "type",
                 node.name,
                 wrap("implements ", join(node.interfaces, " & ")),
                 join(node.directives, " "),
                 block(node.fields),
-            ],
+            ),
             " ",
         )
 
+    @staticmethod
     @add_description
-    def leave_field_definition(self, node, *_args):
+    def leave_field_definition(node: PrintedNode, *_args: Any) -> str:
         args = node.arguments
         args = (
             wrap("(\n", indent(join(args, "\n")), "\n)")
@@ -190,60 +249,67 @@ class PrintAstVisitor(Visitor):
         directives = wrap(" ", join(node.directives, " "))
         return f"{node.name}{args}: {node.type}{directives}"
 
+    @staticmethod
     @add_description
-    def leave_input_value_definition(self, node, *_args):
+    def leave_input_value_definition(node: PrintedNode, *_args: Any) -> str:
         return join(
-            [
+            (
                 f"{node.name}: {node.type}",
                 wrap("= ", node.default_value),
                 join(node.directives, " "),
-            ],
+            ),
             " ",
         )
 
+    @staticmethod
     @add_description
-    def leave_interface_type_definition(self, node, *_args):
+    def leave_interface_type_definition(node: PrintedNode, *_args: Any) -> str:
         return join(
-            [
+            (
                 "interface",
                 node.name,
                 wrap("implements ", join(node.interfaces, " & ")),
                 join(node.directives, " "),
                 block(node.fields),
-            ],
+            ),
             " ",
         )
 
+    @staticmethod
     @add_description
-    def leave_union_type_definition(self, node, *_args):
+    def leave_union_type_definition(node: PrintedNode, *_args: Any) -> str:
         return join(
-            [
+            (
                 "union",
                 node.name,
                 join(node.directives, " "),
                 "= " + join(node.types, " | ") if node.types else "",
-            ],
+            ),
             " ",
         )
 
+    @staticmethod
     @add_description
-    def leave_enum_type_definition(self, node, *_args):
+    def leave_enum_type_definition(node: PrintedNode, *_args: Any) -> str:
         return join(
-            ["enum", node.name, join(node.directives, " "), block(node.values)], " "
+            ("enum", node.name, join(node.directives, " "), block(node.values)), " "
         )
 
+    @staticmethod
     @add_description
-    def leave_enum_value_definition(self, node, *_args):
-        return join([node.name, join(node.directives, " ")], " ")
+    def leave_enum_value_definition(node: PrintedNode, *_args: Any) -> str:
+        return join((node.name, join(node.directives, " ")), " ")
 
+    @staticmethod
     @add_description
-    def leave_input_object_type_definition(self, node, *_args):
+    def leave_input_object_type_definition(node: PrintedNode, *_args: Any) -> str:
         return join(
-            ["input", node.name, join(node.directives, " "), block(node.fields)], " "
+            ("input", node.name, join(node.directives, " "), block(node.fields)), " "
         )
 
+    @staticmethod
     @add_description
-    def leave_directive_definition(self, node, *_args):
+    def leave_directive_definition(node: PrintedNode, *_args: Any) -> str:
         args = node.arguments
         args = (
             wrap("(\n", indent(join(args, "\n")), "\n)")
@@ -254,64 +320,71 @@ class PrintAstVisitor(Visitor):
         locations = join(node.locations, " | ")
         return f"directive @{node.name}{args}{repeatable} on {locations}"
 
-    def leave_schema_extension(self, node, *_args):
+    @staticmethod
+    def leave_schema_extension(node: PrintedNode, *_args: Any) -> str:
         return join(
-            ["extend schema", join(node.directives, " "), block(node.operation_types)],
+            ("extend schema", join(node.directives, " "), block(node.operation_types)),
             " ",
         )
 
-    def leave_scalar_type_extension(self, node, *_args):
-        return join(["extend scalar", node.name, join(node.directives, " ")], " ")
+    @staticmethod
+    def leave_scalar_type_extension(node: PrintedNode, *_args: Any) -> str:
+        return join(("extend scalar", node.name, join(node.directives, " ")), " ")
 
-    def leave_object_type_extension(self, node, *_args):
+    @staticmethod
+    def leave_object_type_extension(node: PrintedNode, *_args: Any) -> str:
         return join(
-            [
+            (
                 "extend type",
                 node.name,
                 wrap("implements ", join(node.interfaces, " & ")),
                 join(node.directives, " "),
                 block(node.fields),
-            ],
+            ),
             " ",
         )
 
-    def leave_interface_type_extension(self, node, *_args):
+    @staticmethod
+    def leave_interface_type_extension(node: PrintedNode, *_args: Any) -> str:
         return join(
-            [
+            (
                 "extend interface",
                 node.name,
                 wrap("implements ", join(node.interfaces, " & ")),
                 join(node.directives, " "),
                 block(node.fields),
-            ],
+            ),
             " ",
         )
 
-    def leave_union_type_extension(self, node, *_args):
+    @staticmethod
+    def leave_union_type_extension(node: PrintedNode, *_args: Any) -> str:
         return join(
-            [
+            (
                 "extend union",
                 node.name,
                 join(node.directives, " "),
                 "= " + join(node.types, " | ") if node.types else "",
-            ],
+            ),
             " ",
         )
 
-    def leave_enum_type_extension(self, node, *_args):
+    @staticmethod
+    def leave_enum_type_extension(node: PrintedNode, *_args: Any) -> str:
         return join(
-            ["extend enum", node.name, join(node.directives, " "), block(node.values)],
+            ("extend enum", node.name, join(node.directives, " "), block(node.values)),
             " ",
         )
 
-    def leave_input_object_type_extension(self, node, *_args):
+    @staticmethod
+    def leave_input_object_type_extension(node: PrintedNode, *_args: Any) -> str:
         return join(
-            ["extend input", node.name, join(node.directives, " "), block(node.fields)],
+            ("extend input", node.name, join(node.directives, " "), block(node.fields)),
             " ",
         )
 
 
-def join(strings: Optional[Collection[str]], separator: str = "") -> str:
+def join(strings: Optional[Strings], separator: str = "") -> str:
     """Join strings in a given collection.
 
     Return an empty string if it is None or empty, otherwise join all items together
@@ -320,7 +393,7 @@ def join(strings: Optional[Collection[str]], separator: str = "") -> str:
     return separator.join(s for s in strings if s) if strings else ""
 
 
-def block(strings: Collection[str]) -> str:
+def block(strings: Strings) -> str:
     """Return strings inside a block.
 
     Given a collection of strings, return a string with each item on its own line,
@@ -352,6 +425,6 @@ def is_multiline(string: str) -> bool:
     return "\n" in string
 
 
-def has_multiline_items(maybe_list: Optional[Collection[str]]):
+def has_multiline_items(maybe_list: Optional[Strings]) -> bool:
     """Check whether one of the items in the list has multiple lines."""
-    return maybe_list and any(is_multiline(item) for item in maybe_list)
+    return any(is_multiline(item) for item in maybe_list) if maybe_list else False
