@@ -5,11 +5,11 @@ from .frozen_error import FrozenError
 
 __all__ = ["FrozenDict"]
 
-K = TypeVar("K")
-T = TypeVar("T", covariant=True)
+KT = TypeVar("KT")
+VT = TypeVar("VT", covariant=True)
 
 
-class FrozenDict(Dict[K, T]):
+class FrozenDict(Dict[KT, VT]):
     """Dictionary that can only be read, but not changed."""
 
     def __delitem__(self, key):
@@ -24,12 +24,12 @@ class FrozenDict(Dict[K, T]):
     def __hash__(self):
         return hash(tuple(self.items()))
 
-    def __copy__(self):
+    def __copy__(self) -> "FrozenDict":
         return FrozenDict(self)
 
     copy = __copy__
 
-    def __deepcopy__(self, memo):
+    def __deepcopy__(self, memo: Dict) -> "FrozenDict":
         return FrozenDict({k: deepcopy(v, memo) for k, v in self.items()})
 
     def clear(self):
